package apirtdatabaseq

import "git.sr.ht/~mil/mobsql"

// DatabaseqParams holds parameters for the given [RTDatabaseq] request.
// There are two fields (FeedIDs and FeedSearchFilter) which may be specified
// which are used in an either/xor-fashion to determine the feeds to query
// the database state of.
//
//   - FeedIDs: Allows you to specify particular feed IDs to query
//     the database status of.
//   - FeedSearchFilter: Allows you to specify a 'search filter' to determine
//     feeds to query the database state of. Using the feed search filter
//     allows you to build out 'search' interfaces to match the loaded
//     Mobility Database for feed ID matches (e.g. see underlying struct
//     Glob field and similar).
type DatabaseqParams struct {
	FeedIDs          []int             `json:"feed_ids"`
	FeedSearchFilter *FeedsearchFilter `json:"feed_searchfilter"`
}

// FeedStatusInfo contains underlying info about a feed returned from [RTDatabaseq]
//
// This is an alias of the underlying [git.sr.ht/~mil/mobsql.FeedStatusInfo].
// Please see underlying type for further details.
type FeedStatusInfo = mobsql.FeedStatusInfo

// DatabaseqResponse holds the response for the given [RTDatabaseq] function.
// Contains information about each feed queried.
//
// This is an alias of the underlying [FeedStatusInfo].
// Please see underlying type for further details.
type DatabaseqResponse = []FeedStatusInfo

// FeedsearchFilter contains a filter specification that can specify a search
// on metadatafields from the Mobility Database.
//
// This is an alias of the underlying [git.sr.ht/~mil/mobsql.FeedsearchFilter]
// Please see underlying type for further details.
type FeedsearchFilter = mobsql.FeedsearchFilter
