package apirtstops

import (
	"git.sr.ht/~mil/mobroute/dbt"
)

// StopsParams contains parameters for the [RTStops] function. This struct
// contains the constraints that will be converted to a SQL query to match
// stops in the DB. Note that multiple feed IDs can be queried at the
// same time as returned stops have stop UIDs. The only required field is
// FeedIDs and all other fields are optional.
//
// If FromCoord is set, MaxDistKm should be set as well as this will limit
// returned stops to only be within the distance specified from the coordinate.
//
// Glob is a wildcard that will text match against stop names and filter
// as such.
//
// NLimit limits the number of returned stops to be the maximum of this value.
type StopsParams struct {
	FeedIDs   []int       `json:"feed_ids" validate:"required,gt=0"`
	FromCoord *[2]float64 `json:"from_coord"`
	MaxDistKM *float64    `json:"from_maxdistkm"`
	Glob      string      `json:"glob"`
	NLimit    int         `json:"n_limit"`
}

// StopsResponse is the response for the [RTStops] function that contains
// the matched stops. See the field Stops which contains an array of
// [git.sr.ht/~mil/mobroute/dbt.StopWalk].
type StopsResponse struct {
	Stops *[]dbt.StopWalk `json:"stops,omitempty"`
}
