package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"os"

	"git.sr.ht/~mil/mobroute"
)

var (
	subcmdDatabaseqExamples = map[string]string{
		"Determine the status of single GTFS feed":                   `mobroute databaseq -p '{"feed_ids": [767]}'`,
		"Determine the status of multiple GTFS feeds":                `mobroute databaseq -p '{"feed_ids": [767, 516]}'`,
		"Determine the status of a particular search for GTFS feeds": `mobroute databaseq -p '{"feed_searchfilter": {"glob": "new york"}}'`,
	}
)

const (
	subcmdDatabaseqDesc = `
The databaseq subcommand queries the underlying SQLite database to
determine the loaded/computed status along with metadata of particular
GTFS feeds. This subcommand can be run after the database command (and
before route, schedule, stops) commands to ensure data is loaded/computed.
It can also be used to arbitrarily query metadata via the search filter
ability.

See Mobsql (http://git.sr.ht/~mil/mobsql) for in-depth documentation
on functionality (which this subcommand is a passthrough to).
	`

	subcmdDatabaseqDocParams = `
JSON Parameters for the databaseq subcommand.

Optional databaseq parameters:
  - feed_ids: Array of integer feed IDs to use for databaseq request (ex: [767])
  - feed_searchfilter: Search filter object (ex: {"glob": "new york"})
`
)

func cmdDatabaseq(flagDatabaseqSet *flag.FlagSet, cfg *mobroute.MobrouteRuntimeConfig, flagDatabaseqParams *string) {
	var databaseqParams mobroute.DatabaseqParams
	flagDatabaseqSet.Usage = usageFunc("databaseq", flagDatabaseqSet, subcmdDatabaseqDesc, subcmdDatabaseqExamples)
	flagDatabaseqSet.Parse(os.Args[2:])
	if *flagDatabaseqParams == "" {
		flagDatabaseqSet.Usage()
	} else if err := json.Unmarshal([]byte(*flagDatabaseqParams), &databaseqParams); err != nil {
		errorDie(os.Args[1], *flagDatabaseqParams, err)
	} else if mobrouteRuntime, err := mobroute.RTInitialize(cfg); err != nil {
		errorDie(os.Args[1], *flagDatabaseqParams, err)
	} else if commandData, err := mobroute.RTDatabaseq(mobrouteRuntime, &databaseqParams); err != nil {
		errorDie(os.Args[1], *flagDatabaseqParams, err)
	} else if jsonBytes, err := json.MarshalIndent(commandData, "", "  "); err != nil {
		errorDie(os.Args[1], *flagDatabaseqParams, err)
	} else {
		fmt.Println(string(jsonBytes))
	}
}
