package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"os"

	"git.sr.ht/~mil/mobroute"
)

var (
	subcmdStopsExamples = map[string]string{
		"Brussels Stops (all) Example":                `mobroute stops -p '{"feed_ids": [1088]}'`,
		"Krakow Stops (all) Example":                  `mobroute stops -p '{"feed_ids": [1270]}'`,
		"NYC Stops (all) Example":                     `mobroute stops -p '{"feed_ids": [516]}'`,
		"Prague Stops (with distance filter) Example": `mobroute stops -p '{"feed_ids": [767], "from_coord": [49.620842, 13.576023], "from_maxdistkm": 4}'`,
	}
)

const (
	subcmdStopsDesc = `
The stops subcommand allows you to query stops for a given set of feed IDs
optionally bounded to a km distance from a specific coordinate. If km distance
or coordinate is unspecified, all stops for the feed are returned.

Before running this subcommand, the target feed_ids should be loaded
& computed in the database.
`
	subcmdStopsDocParams = `
JSON Parameters for stops subcommand.

Required stops parameters:
  - feed_ids: Array of integer feed IDs to use for routing requests (ex: [1088, 516])

Optional stops parameters:
	- n_limit: Limit number of stops (if unset no limit - e.g. -1)
	- glob: String to match stop_name against to filter results (if unset no glob is applied)
	- from_coord: Latlon coordinate pair to limit (Coordinate to bound results from if max_km_distance provided)
	- from_maxdistkm: Maximum km distance from  (Maximum distance km)
`
)

func cmdStops(flagStopsSet *flag.FlagSet, cfg *mobroute.MobrouteRuntimeConfig, flagStopsParams *string) {
	var stopsParams mobroute.StopsParams
	flagStopsSet.Usage = usageFunc("stops", flagStopsSet, subcmdStopsDesc, subcmdStopsExamples)
	flagStopsSet.Parse(os.Args[2:])
	if *flagStopsParams == "" {
		flagStopsSet.Usage()
	} else if err := json.Unmarshal([]byte(*flagStopsParams), &stopsParams); err != nil {
		errorDie(os.Args[1], *flagStopsParams, err)
	} else if mobrouteRuntime, err := mobroute.RTInitialize(cfg); err != nil {
		errorDie(os.Args[1], *flagStopsParams, err)
	} else if commandData, err := mobroute.RTStops(mobrouteRuntime, &stopsParams); err != nil {
		errorDie(os.Args[1], *flagStopsParams, err)
	} else if jsonBytes, err := json.MarshalIndent(commandData, "", "  "); err != nil {
		errorDie(os.Args[1], *flagStopsParams, err)
	} else {
		fmt.Println(string(jsonBytes))
	}
}
