package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"log"
	"os"
	"runtime/pprof"
	"strings"

	"git.sr.ht/~mil/mobroute"
)

func errorDie(subcmd, req string, err error) {
	if err != nil {
		log.Printf("Request failed (use 'mobroute %s -h' for proper usage)", subcmd)
		log.Printf("Failed JSON request buffer: %s", req)
		log.Fatal(err)
	}
	os.Exit(1)
}

func globalsSetup() (func(), *mobroute.MobrouteRuntimeConfig) {
	// Setup pprof debugging
	var (
		envPprofFile   = os.Getenv("MOBROUTE_PPROF")
		envMobrouteCfg = os.Getenv("MOBROUTE_CFG")

		pprofCompleteFunc                                 = func() {}
		mobrouteCfg       *mobroute.MobrouteRuntimeConfig = nil
	)

	if envPprofFile != "" {
		if f, err := os.Create(envPprofFile); err != nil {
			log.Fatal(err)
		} else {
			pprof.StartCPUProfile(f)
			pprofCompleteFunc = pprof.StopCPUProfile
		}
	}
	if envMobrouteCfg != "" {
		if err := json.Unmarshal([]byte(envMobrouteCfg), &mobrouteCfg); err != nil {
			log.Fatalf("Passed env var MOBROUTE_CFG but failed to unmarhsal JSON (%s): %v", envMobrouteCfg, err)
		}
	}

	return pprofCompleteFunc, mobrouteCfg
}

func usageFunc(cmd string, flagSet *flag.FlagSet, cmdDesc string, cmdExamples map[string]string) func() {
	return func() {
		fmt.Fprintf(os.Stderr, "Usage of %s %s:\n\n", os.Args[0], cmd)
		fmt.Fprintf(os.Stderr, "Description:\n  %s\n\n", strings.Trim(strings.Replace(cmdDesc, "\n", " ", -1), " "))
		fmt.Fprintf(os.Stderr, "Flags: \n")
		flagSet.PrintDefaults()
		fmt.Fprintf(os.Stderr, "\nExamples:\n")
		for exName, exCmd := range cmdExamples {
			fmt.Fprintf(os.Stderr, "  %s:\n    %s\n", exName, exCmd)
		}
		fmt.Fprintf(os.Stderr, "\n")
	}
}
