with
	input_coord as (select ? as lat, ? as lon),
	dists as (
		select
			distinct s.feed_id, vsp.feed_id || '_' || vsp.parent_station as stop_uid,
			s.stop_name,
			s.stop_lat,
			s.stop_lon,
			6371.0 * acos(
				sin(input_coord.lat * pi() / 180.0) * sin(s.stop_lat * pi() / 180.0) +
				cos(input_coord.lat * pi() / 180.0) * cos(s.stop_lat * pi() / 180.0) *
				cos((s.stop_lon - input_coord.lon) * pi() / 180.0)
			) as dist_km_a
		from
			input_coord
			join stops s on 1 = 1
			join _vstopparent vsp on s.feed_id = vsp.feed_id and vsp.parent_station = s.stop_id
			order by dist_km_a asc
	)
select
  stop_uid,
  stop_name,
  stop_lat,
  stop_lon,
  iif(dist_km_a > ?, null, dist_km_a) as dist_km
from dists
  where feed_id in (%s)
  and (length(?) = 0 or lower(stop_uid || ' ' || stop_name) glob lower('*' || ? || '*'))
  %s
order by dist_km asc
limit ?
