/*
Mobroute - Load Custom GTFS Example:
  - Uses RTDatabase to load a custom GTFS feed
  - GTFS feed can be either a file:// or http:// URI
  - Uses RTDatabase to compute GTFS-derived tables
  - Note: for custom feeds, the feed ID must be a negative user provided integer
*/
package main

import (
	"log"

	"git.sr.ht/~mil/mobroute"
)

func main() {
	var (
		feedID                = -43
		gtfsURI               = "http://web.mta.info/developers/data/nyct/subway/google_transit.zip"
		dbParamsLoadCustom    = mobroute.DatabaseParams{Op: "loadcustomgtfs", FeedIDs: []int{feedID}, LoadCustomGTFSURI: gtfsURI}
		dbParamsComputeCustom = mobroute.DatabaseParams{Op: "compute", FeedIDs: []int{feedID}}
	)

	if mobrouteRuntime, err := mobroute.RTInitialize(&mobroute.MobrouteRuntimeConfig{}); err != nil {
		log.Fatal("Failed to initialize Mobroute Runtime: ", err)
	} else if _, err := mobroute.RTDatabase(mobrouteRuntime, &dbParamsLoadCustom); err != nil {
		log.Fatal("Failed to loadcustomgtfs: ", err)
	} else if _, err := mobroute.RTDatabase(mobrouteRuntime, &dbParamsComputeCustom); err != nil {
		log.Fatal("Failed to compute: ", err)
	}
}
