/*
Mobroute - Route Example:
  - Uses RTDatabase to load a custom GTFS feed
  - GTFS feeds can be referenced at http://mdb.lrdu.org
  - Uses RTDatabase to compute GTFS-derived tables
  - Calculates route using CSA via RTRoute
  - Show example of using the mapurl & legs output formats
  - Legs format can be used for trip planning showing each leg / step of route
  - Mapurl is viewable link showing the route on an OSM / leaflet map
*/
package main

import (
	"encoding/json"
	"log"

	"git.sr.ht/~mil/mobroute"
)

func main() {
	var (
		feedID          = 1088
		fromLatLon      = [2]float64{50.85728, 4.351426}
		toLatLon        = [2]float64{50.83214, 4.350534}
		dbParamsLoad    = mobroute.DatabaseParams{Op: "loadmdbgtfs", FeedIDs: []int{feedID}}
		dbParamsCompute = mobroute.DatabaseParams{Op: "compute", FeedIDs: []int{feedID}}
		routeParams     = mobroute.RouteParams{
			FeedIDs:            []int{feedID},
			From:               fromLatLon,
			To:                 toLatLon,
			TransferCategories: []string{"f", "i"},
			OutputFormats:      []string{"legs", "mapurl"},
		}
	)

	if mobrouteRuntime, err := mobroute.RTInitialize(&mobroute.MobrouteRuntimeConfig{}); err != nil {
		log.Fatal("Failed to initialize Mobroute Runtime: ", err)
	} else if _, err := mobroute.RTDatabase(mobrouteRuntime, &dbParamsLoad); err != nil {
		log.Fatal("Failed to loadmdbgtfs: ", err)
	} else if _, err := mobroute.RTDatabase(mobrouteRuntime, &dbParamsCompute); err != nil {
		log.Fatal("Failed to compute: ", err)
	} else if routeResult, err := mobroute.RTRoute(mobrouteRuntime, &routeParams); err != nil {
		log.Fatal("Failed to route: ", err)
	} else {
		jsonLegs, _ := json.Marshal(*routeResult.RouteLegs)
		log.Println("Route Legs:", string(jsonLegs))
		log.Println("Viewable map URL:", *routeResult.MapURL)
	}
}
