/*
Mobroute - Schedule Example:
  - Uses RTDatabase to load a custom GTFS feed
  - GTFS feeds can be referenced at http://mdb.lrdu.org
  - Uses RTDatabase to compute GTFS-derived tables
  - Uses RTSchedule to get upcoming departures within next 3000 minutes
  - Note stop UIDs can be obtained by using RTStops (see stops example)
*/
package main

import (
	"encoding/json"
	"log"

	"git.sr.ht/~mil/mobroute"
)

func main() {
	var (
		feedID          = 1088
		stopUID         = "1088_61"
		dbParamsLoad    = mobroute.DatabaseParams{Op: "loadmdbgtfs", FeedIDs: []int{feedID}}
		dbParamsCompute = mobroute.DatabaseParams{Op: "compute", FeedIDs: []int{feedID}}
		scheduleParams  = mobroute.ScheduleParams{
			FeedIDs:    []int{feedID},
			StopUIDs:   []string{stopUID},
			MaxSeconds: 60 * 3000,
		}
	)

	if mobrouteRuntime, err := mobroute.RTInitialize(&mobroute.MobrouteRuntimeConfig{}); err != nil {
		log.Fatal("Failed to initialize Mobroute Runtime: ", err)
	} else if _, err := mobroute.RTDatabase(mobrouteRuntime, &dbParamsLoad); err != nil {
		log.Fatal("Failed to loadmdbgtfs: ", err)
	} else if _, err := mobroute.RTDatabase(mobrouteRuntime, &dbParamsCompute); err != nil {
		log.Fatal("Failed to compute: ", err)
	} else if schedule, err := mobroute.RTSchedule(mobrouteRuntime, &scheduleParams); err != nil {
		log.Fatal("Failed to get schedule: ", err)
	} else {
		jsonSchedule, _ := json.Marshal(schedule.Schedule)
		log.Printf("Schedule: %s\n", jsonSchedule)
	}
}
