/*
Mobroute - Stops Example:
  - Uses RTDatabase to load a custom GTFS feed
  - GTFS feeds can be referenced at http://mdb.lrdu.org
  - Uses RTDatabase to compute GTFS-derived tables
  - Uses RTStops to get 10 valid stops for the given feed
  - If you specify input coordinate lat/lon you can get stops within a distance
  - If unspecified, all stops for given feed(s) will be returned
*/
package main

import (
	"encoding/json"
	"log"

	"git.sr.ht/~mil/mobroute"
)

func main() {
	var (
		feedID          = 1088
		maxDistKM       = 5.0
		dbParamsLoad    = mobroute.DatabaseParams{Op: "loadmdbgtfs", FeedIDs: []int{feedID}}
		dbParamsCompute = mobroute.DatabaseParams{Op: "compute", FeedIDs: []int{feedID}}
		stopParams      = mobroute.StopsParams{
			FeedIDs:   []int{feedID},
			FromCoord: &[2]float64{50.817588, 4.26531},
			MaxDistKM: &maxDistKM,
			NLimit:    10,
		}
	)

	if mobrouteRuntime, err := mobroute.RTInitialize(&mobroute.MobrouteRuntimeConfig{}); err != nil {
		log.Fatal("Failed to initialize Mobroute Runtime: ", err)
	} else if _, err := mobroute.RTDatabase(mobrouteRuntime, &dbParamsLoad); err != nil {
		log.Fatal("Failed to loadmdbgtfs: ", err)
	} else if _, err := mobroute.RTDatabase(mobrouteRuntime, &dbParamsCompute); err != nil {
		log.Fatal("Failed to compute: ", err)
	} else if stops, err := mobroute.RTStops(mobrouteRuntime, &stopParams); err != nil {
		log.Fatal("Failed to get stops: ", err)
	} else {
		jsonStops, _ := json.Marshal(stops.Stops)
		log.Printf("Stops: %s\n", jsonStops)
	}
}
