/*
Mobroute - Update Mobility Database GTFS Feed Example:
  - Demonstrates updating of a Mobility Database GTFS Feed (as specified by feed ID)
  - Feed IDs can be referenced from http://mdb.lrdu.org
  - Uses RTDatabase to load the feed (downloads GTFS data & imports to SQLite)
  - Uses RTDatabase to computed GTFS-derived tables
  - Note use of LoadMDBGTFSUpdate param
  - Uses direct agency GTFS URL rather then default CI-mirrored GTFS URL
*/
package main

import (
	"log"

	"git.sr.ht/~mil/mobroute"
)

func main() {
	var (
		feedID       = 1088
		dbParamsLoad = mobroute.DatabaseParams{
			Op:                    "loadmdbgtfs",
			FeedIDs:               []int{feedID},
			LoadMDBGTFSDirectURLs: true, // E.g. use agency URL (not default CI-mirrored GTFS)
			LoadMDBGTFSUpdate:     true, // E.g. ignore caching systems, ensure latest GTFS URLs used
		}
		dbParamsCompute = mobroute.DatabaseParams{Op: "compute", FeedIDs: []int{feedID}}
	)

	if mobrouteRuntime, err := mobroute.RTInitialize(&mobroute.MobrouteRuntimeConfig{}); err != nil {
		log.Fatal("Failed to initialize Mobroute Runtime: ", err)
	} else if _, err := mobroute.RTDatabase(mobrouteRuntime, &dbParamsLoad); err != nil {
		log.Fatal("Failed to load: ", err)
	} else if _, err := mobroute.RTDatabase(mobrouteRuntime, &dbParamsCompute); err != nil {
		log.Fatal("Failed to compute: ", err)
	}
}
