package mobroute

// Note to reader:
// This file is an **autogenerated** aliasing amaglimation from all packages in api/
//
// Generating the master aliasing amaglimation allows for two purposes:
//   1. Allows users to simply import the top-level sr.ht/~mil/mobroute package
//      for full public library usage.
//   2. Development-wise, each library function and its associated types are
//      guaranteed to be discrete & isolated from one another by design.
//
// DO NOT manually edit this file, it may be regenerated by using:
// scripts/generate_mobroutelibgo.sh > mobroute_lib.go

import (
	"git.sr.ht/~mil/mobroute/api/apirtdatabase"
	"git.sr.ht/~mil/mobroute/api/apirtdatabaseq"
	"git.sr.ht/~mil/mobroute/api/apirtinit"
	"git.sr.ht/~mil/mobroute/api/apirtroute"
	"git.sr.ht/~mil/mobroute/api/apirtschedule"
	"git.sr.ht/~mil/mobroute/api/apirtstops"
)

// Package: apirtinit
// ===========

// MobsqlRuntimeConfig alias for [git.sr.ht/~mil/mobsql.RuntimeConfig].
// In most normal cases where this is seen in [MobrouteRuntimeConfig]
// this should just be left as nil. The case when it makes sense
// to configure underlying the underlying [Mobsql] config is if you
// are interested in changing logging and other underlying schema properties
// for Mobsql. See underlying origin [git.sr.ht/~mil/mobsql.RuntimeConfig]
// doc for more details.
//
// [Mobsql]: https://mr.lrdu.org/mobsql
type MobsqlRuntimeConfig = apirtinit.MobsqlRuntimeConfig

// MobrouteRuntime contains the database connection and the
// [git.sr.ht/~mil/mobsql.MobsqlRuntime]. This may also contain other
// properties relevant for global Mobroute operation in the future.
// The database connection points to a local SQLite file and the Mobsql
// runtime connection generally will reference the same DB file.
// The reason for storing the DB separately from Mobsql's runtime (and not
// just using Mobsql's own DB connection) is that Mobroute itself uses
// the DB field for RO operations only; all RW functionality happens
// through Mobsql / the MobsqlRuntime.
//
// This struct is passed to all other library functions and should be
// initialized by using [RTInitialize].
type MobrouteRuntime = apirtinit.MobrouteRuntime

// MobrouteRuntimeConfig contains the the underlying
// [git.sr.ht/~mil/mobsql/MobsqlRuntimeConfig] and potentially in the future
// some other Mobroute-specific properties for the common DB connection and
// Mobroute params config. This is used by [RTInitialize] and you can use
// this struct to configure various properties around logging and [Mobsql]
// initialization.
//
// The Logger field can be very helpful in redirecting log messages to see
// more verbose output for functions then just the returned type / error.
//
// An empty configuration (e.g. nil values) for all properties is entirely
// valid and defaults will be configured in that case.
//
// [Mobsql]: https://mr.lrdu.org/mobsql
type MobrouteRuntimeConfig = apirtinit.MobrouteRuntimeConfig

// RTInitialize initializes a new [MobrouteRuntime] which contains a
// passthrough to the [git.sr.ht/~mil/mobsql.MobsqlRuntime], the SQLite DB
// connection, and some additional metadata. On initialization
// this function: bootstraps the DB (e.g. creating SQL tables, views, indexes),
// create Mobsql connection / initialization, and fetches the Mobility Database
// CSV and imports to DB.
//
// All other library functionality depend on the returned [MobrouteRuntime].
func RTInitialize(params *MobrouteRuntimeConfig) (*MobrouteRuntime, error) {
	return apirtinit.RTInitialize(params)
}

// ===========

// Package: apirtdatabase
// ===========

// RTDatabase executes database RW manipulation allowing for: loading GTFS
// archives (via [Mobility Database] or custom URIs), creating computed tables,
// removing GTFS archives from DB, and more. Takes [DatabaseParams] with
// given database manipulation request and returns [DatabaseResponse] which
// indicates success or failure. Under-the-hood, this is a thin passthrough
// to the major RW functionality in [Mobsql], allowing the user to manipulate
// the underlying database.
//
// Note, the related [RTDatabaseq] allows for querying (RO) functionality on
// the underlying database.
//
// [Mobsql]: https://mr.lrdu.org/mobsql
// [Mobility Database]: https://mobilitydatabase.org/
func RTDatabase(runtime *MobrouteRuntime, params *DatabaseParams) (*DatabaseResponse, error) {
	return apirtdatabase.RTDatabase(runtime, params)
}

// DatabaseParams holds parameters for the given [RTDatabase] request. The
// [RTDatabase] function and the options in these params are a thin
// passthrough to [Mobsql] (which should be referenced for in-depth doc
// on functionality).
//
// FeedIDs and Op are always required.
//
// Op should be one of
//   - loadmdbgtfs: Load the GTFS for the feed IDs from the Mobility Database
//     to the SQLite DB.
//   - loadcustomgtfs: Load the GTFS for the provided URI to the local SQLite
//     DB.
//   - compute: Create computed tables entries for the given feed IDs.
//   - purge_gtfs: Remove origin GTFS tables for the given feed IDs.
//   - purge_computed: Remove computed tables for the given feed IDs.
//   - purge_all: Remove both GTFS & computed tables entries for the
//     given feed IDs.
//
// FeedIDs are the [Mobility Database] feed ids (or custom feed ids). In the case
// of using a custom feed ID (for loadcustomgtfs), the feed ID must be
// user-specified *negative* integer. For more information see the [Feed IDs doc].
//
// LoadCustomGTFSURI is only used to define the URI when using loadcustomgtfs op
// and may refer to either an HTTP(s) (http://) URL for a remote GTFS archive
// or a File URI (file://) for a local GTFS archive to load.
//
// LoadMDBGTFSUpdate is only used when using the loadmdbgtfs op and determines
// whether the feed should be force 'full updated'. This involves
// re-downloading the Mobility DB prior to GTFS load & ignoring all caching
// systems. This is helpful in the case of feed updates.
//
// LoadMDBGTFSDirectURLs is only used when using the loadmdbgtfs op and
// determines whether the 'direct' agency URL should be used when downloading
// GTFS archives. By default / if false, a cloud-mirrored GTFS URL (provided
// by [Mobility Database]'s CI is used). The default cloud mirrored URL is
// more 'reliable' but can go out-of-date.
//
// Important Note: After running a `loadmdbgtfs` or `loadcustomgtfs` op, in
// most cases you will want to run a `compute` op. Computed data is based
// on origin GTFS data and each time updating GTFS data you should re-compute
// derived tables with `compute` op subsequently.
//
// [Mobility Database]: https://mobilitydatabase.org/
// [Mobsql]: https://mr.lrdu.org/mobsql
// [Feed IDs doc]: https://mr.lrdu.org/mobroute/doc/doc_gtfsfeeds/
type DatabaseParams = apirtdatabase.DatabaseParams

// Databaseq holds the response for the given [RTDatabase] function. Contains
// information about whether the given [RTDatabase] call was successful and
// contains metadata about effected feeds.
//
// DatabaseResponse is an alias of [git.sr.ht/~mil/mobsql.FeedOpResult].
// Please see underlying type for further details.
type DatabaseResponse = apirtdatabase.DatabaseResponse

// ===========

// Package: apirtdatabaseq
// ===========

// RTDatabaseq queries database RO to determine GTFS feed loaded status
// (resulting from [RTDatabase]) & shows metadata for unloaded and loaded
// feeds. Takes the query params in [DatabaseqParams] and returns resulting
// set of feed data in [DatabaseqResponse]. This is an RO operation as
// opposed to [RTDatabase] which may manipulate the database as RW.
// This function is helpful for querying whether a load operation, or compute
// operation was successful and also determining the current database state
// and the state of GTFS feeds. Under-the-hood, this is a thin passthrough to
// the major RO functionality in [Mobsql], allowing the user to query the
// underlying database. See [DatabaseqParams] for details on params.
//
// Note, the related [RTDatabase] allows for manipulation (RW) functionality on
// the underlying database.
//
// [Mobsql]: https://mr.lrdu.org/mobsql
//
// [Mobility Database]: https://mobilitydatabase.org/
func RTDatabaseq(runtime *MobrouteRuntime, params *DatabaseqParams) (*DatabaseqResponse, error) {
	return apirtdatabaseq.RTDatabaseq(runtime, params)
}

// DatabaseqParams holds parameters for the given [RTDatabaseq] request.
// There are two fields (FeedIDs and FeedSearchFilter) which may be specified
// which are used in an either/xor-fashion to determine the feeds to query
// the database state of.
//
//   - FeedIDs: Allows you to specify particular feed IDs to query
//     the database status of.
//   - FeedSearchFilter: Allows you to specify a 'search filter' to determine
//     feeds to query the database state of. Using the feed search filter
//     allows you to build out 'search' interfaces to match the loaded
//     Mobility Database for feed ID matches (e.g. see underlying struct
//     Glob field and similar).
type DatabaseqParams = apirtdatabaseq.DatabaseqParams

// FeedStatusInfo contains underlying info about a feed returned from [RTDatabaseq]
//
// This is an alias of the underlying [git.sr.ht/~mil/mobsql.FeedStatusInfo].
// Please see underlying type for further details.
type FeedStatusInfo = apirtdatabaseq.FeedStatusInfo

// DatabaseqResponse holds the response for the given [RTDatabaseq] function.
// Contains information about each feed queried.
//
// This is an alias of the underlying [FeedStatusInfo].
// Please see underlying type for further details.
type DatabaseqResponse = apirtdatabaseq.DatabaseqResponse

// FeedsearchFilter contains a filter specification that can specify a search
// on metadatafields from the Mobility Database.
//
// This is an alias of the underlying [git.sr.ht/~mil/mobsql.FeedsearchFilter]
// Please see underlying type for further details.
type FeedsearchFilter = apirtdatabaseq.FeedsearchFilter

// ===========

// Package: apirtstops
// ===========

// RTStops queries database for GTFS stops near a given geocoordinate or all stops.
// Takes the constraints in [StopsParams] and returns the matched
// stops as [StopsResponse]. This function can be very helpful both to
// build out a 'nearby stops' functionality in end-applications or also
// the return stop UIDs may be used by [RTSchedule] which can show
// upcoming departures for each stop.
//
// Note: provided feed IDs in [StopsParams] should first be loaded and computed via [RTDatabase].
func RTStops(runtime *MobrouteRuntime, params *StopsParams) (*StopsResponse, error) {
	return apirtstops.RTStops(runtime, params)
}

// StopsParams contains parameters for the [RTStops] function. This struct
// contains the constraints that will be converted to a SQL query to match
// stops in the DB. Note that multiple feed IDs can be queried at the
// same time as returned stops have stop UIDs. The only required field is
// FeedIDs and all other fields are optional.
//
// If FromCoord is set, MaxDistKm should be set as well as this will limit
// returned stops to only be within the distance specified from the coordinate.
//
// Glob is a wildcard that will text match against stop names and filter
// as such.
//
// NLimit limits the number of returned stops to be the maximum of this value.
type StopsParams = apirtstops.StopsParams

// StopsResponse is the response for the [RTStops] function that contains
// the matched stops. See the field Stops which contains an array of
// [git.sr.ht/~mil/mobroute/dbt.StopWalk].
type StopsResponse = apirtstops.StopsResponse

// ===========

// Package: apirtschedule
// ===========

// RTSchedule queries the database for upcoming schedule departures for a
// given set of stop UIDs. Takes the departure schedule constraints defined
// in [ScheduleParams] and returns departures within the timeframe in
// a [ScheduleResponse]. Note: the logic used in this function handles
// timezones & operates identically to routing calls in terms of schedule
// extraction. This function is a good way to determine upcoming departures
// for a given set of stops. The stop UIDs may be determined using the [RTStops]
// function.
//
// Note: provided feed IDs in [ScheduleParams] should first be loaded and computed via [RTDatabase].
func RTSchedule(runtime *MobrouteRuntime, params *ScheduleParams) (*ScheduleResponse, error) {
	return apirtschedule.RTSchedule(runtime, params)
}

// ScheduleParams defines the parameters for the [RTSchedule] function.
// The fields FeedIDs, StopUIDs, and MaxSeconds are required. Note that
// StopUIDs may be determined using the [RTStops] function.
//
// The optional Time field allows you define a custom departure time; if
// unset it is assumed to use the current system time (now) for departure.
// The MaxSeconds field is bound to the given time.
type ScheduleParams = apirtschedule.ScheduleParams

// ScheduleDeparture represents a single departure (correlating under-the-hood)
// to an entry in the GTFS stop_times table. This type contains some extra
// information about each departure as well such as the route short / long
// name, trip headsign, and departure time. Departure times are provided as
// UTC.
type ScheduleDeparture = apirtschedule.ScheduleDeparture

// ScheduleResponse represents the response for the [RTSchedule] function
// containing upcoming departures. See the field Schedule which is an
// array of [ScheduleDeparture] indicating potential departures.
type ScheduleResponse = apirtschedule.ScheduleResponse

// ===========

// Package: apirtroute
// ===========

// RTRoute performs point-to-point routing calculations via the
// [Connection Scan Algorithm] based on GTFS data extracted from database
// queries. Effected by routing constraints defined in [RouteParams] and
// returns a [RouteResponse]. The entire routing flow from extracting from
// the DB the required data for CSA, to running the CSA algorithm, to
// going back to the DB to pull a few extra fields for presentation,
// and finally formatting the data into route legs for easy consumption is handled
// by this function. The underlying methodology used for for routing can
// be understood by reading the [Connection Scan Algorithm] paper which this
// function is an implementation of. The [routing dataflow pattern diagram] can
// be helpful in understanding overall dataflow.
//
// Note: provided feed IDs in [RouteParams] should first be loaded and computed via [RTDatabase].
//
// [Connection Scan Algorithm]: https://i11www.iti.kit.edu/extra/publications/dpsw-isftr-13.pdf
// [routing dataflow pattern diagram]: https://mr.lrdu.org/mobroute/doc/doc_diagrams/#routing-dataflow-pattern
func RTRoute(runtime *MobrouteRuntime, params *RouteParams) (*RouteResponse, error) {
	return apirtroute.RTRoute(runtime, params)
}

// RouteParams contains the parameters for a given route request for [RTRoute].
// The FeedIDs, From, To, TransferCategories, and Output format fields are
// required. Other fields are optional and will be filled in with default
// values if unset.
//
//   - Extended information about params (tunables) are available in the [Tunables Doc].
//   - Extended information about output formats are available in the [Output Formats Doc].
//
// [Tunables Doc]: https://mr.lrdu.org/mobroute/doc/routing_tunables/
// [Output Formats Doc]: https://mr.lrdu.org/mobroute/doc/routing_output_formats/
type RouteParams = apirtroute.RouteParams

// RouteResponse is the response for a successful [RTRoute] routing request.
//
// Each field may optionally be present depending on if [RouteParams]'s
// OutputFormats is set for the associated output format.
//
// The [Output Formats Doc] has extended information on each output format,
// a basic overview of each format is as follows:
//   - RouteLegs: Contains an array of 'legs' for each segment of a calculated
//     trip.
//   - GeoJSON: GeoJSON created from the RouteLegs, less information then
//     legs but simpler to integrate with other apps that support GeoJSON.
//   - MapURL: A link to the GeoJSON embeded in hash on [geojson.lrdu.org].
//   - Request: Origin [RouteParams] directly embedded for simpler testing
//     and replication of request.
//   - RouteDiagnostics: The [RouteDiagnostics] contains some simple
//     diagnostics information about the request.
//   - Connections: Array of raw [git.sr.ht/~mil/mobroute/dbt.ConnectionVerbose]
//     that were used for the input to the CSA algorithm. This can be helpful
//     for spot debugging.
//
// [Output Formats Doc]: https://mr.lrdu.org/mobroute/doc/routing_output_formats/
// [geojson.lrdu.org]: https://geojson.lrdu.org
type RouteResponse = apirtroute.RouteResponse

// RouteDiagnostics contains diagnostic information for a routing request
// measuring number of seconds for specific parts of the pipeline,
// memory usage, and meta route information.
//
//   - The PerfS fields indicate the amount of time taken for each stage.
//   - The PerfNMem fields indicate the length of the arrays for each SQL
//     query type extracted
//   - The PerfOptimizedFeeds fields indicated which feed IDs were actually
//     queries (as this is optimized earlier in the pipeline).
//   - The Routeinfo fields contain various calculated route metalevel
//     diagnostics.
type RouteDiagnostics = apirtroute.RouteDiagnostics

// RouteLeg represents a 'leg' in a route which may be one of: transfer,
// walk, or trip type (see the LegType field).
//
// When the LegType is equal to "trip", all fields prefixed with Trip* will
// be non-nil / present. A trip leg represents one boarding of a segment
// of a trip (e.g. a single bus route, or a single train route etc.).
// Underlying this is flagged by a single trip ID sourced from the
// stop_times GTFS table.
//
// When the LegType is equal to "walk", all fields prefixed with Walk* will
// be non-nil / present. A walk leg represents the segment of time walking
// to the origin station (e.g. with the first trip); or walking from the
// destination station (e.g. with the last trip). A walk has a correlated
// distance KM and the walk to/from fields will contain the relevant
// stop IDs. The walk KM is automatically calculated from the input
// constraints for walk km / hr.
//
// When the LegType is equal to "transfer", all fields prefixed with Transfer*
// will be non-nil / present. A transfer leg represents the segment of time
// to get from one trip leg to another trip leg. Transfers can be sourced
// from a number of places (e.g. either implicit, feed, or generated) and
// always have an associated duration. The field TransferDdwell indicates
// the amount of inactive time in the transfer (waiting / dwelling); while
// TransferActive indicates active time walking getting from one stop
// to another.
type RouteLeg = apirtroute.RouteLeg

// RouteStopDetails represents a stop within a trip. This allows more
// fine-grained detailed into each 'stop' in a common trip. Ultimately
// this correlates to a single connection in the table _ctconn and can
// be referenced by using the field ConnOID.
type RouteStopDetails = apirtroute.RouteStopDetails

// ===========
