#!/usr/bin/env sh
cd "$(realpath "$(dirname "$0")")"/.. || exit 1

publicexports() {
	# Given go src in stdin, aliases each type based on the PKG ($1) and
	# each func is wrapped using the new aliased types
	PKG="$1"
	sed -E '/^[\t ]/d' |
		sed '/^$/d' |
		sed -E '/^package/,$!d' |
		sed '/^package\|^import\|^)\|^}/d'  |
		sed -E '/^func [(a-z]/d' |
		sed -E '/^type [a-z]/d' |
		sed '/^func/ s/$/\n/' |
		sed '/^type/ s/$/\n/' |
		sed "s/^type \([a-zA-Z0-9_]*\).*/type \1 = $PKG.\1/g" |
    sed -E "s#^func ([^(]+)[(]([^)]+)[)].*#\0\n  return $PKG.\1(\2)\n}#g" |
		sed -E '/return/ {s/[^, ]*,/,/} /return/ {s/[^) ]+)/)/}' |
		sed -E '/return/ s/ , /, /g' |
		sed -E '/func/ s/apirtinit.MobrouteRuntime/MobrouteRuntime/g'
}

publicexportsdir() {
	DIR="$1"
	echo "// Package: $DIR"
	echo "// ==========="
	echo
	ls api/$DIR | grep -Ev 'package.*.go' | grep .go$ | xargs -IG cat api/$DIR/G | publicexports $DIR
	echo "// ==========="
	echo
	echo
}

generate() {
	echo '
		package mobroute
		// Note to reader:
		// This file is an **autogenerated** aliasing amaglimation from all packages in api/
		//
		// Generating the master aliasing amaglimation allows for two purposes:
		//   1. Allows users to simply import the top-level sr.ht/~mil/mobroute package
		//      for full public library usage.
		//   2. Development-wise, each library function and its associated types are
		//      guaranteed to be discrete & isolated from one another by design.
		// 
		// DO NOT manually edit this file, it may be regenerated by using:
		// scripts/generate_mobroutelibgo.sh > mobroute_lib.go

		import (
			"git.sr.ht/~mil/mobroute/api/apirtinit"
			"git.sr.ht/~mil/mobroute/api/apirtdatabase"
			"git.sr.ht/~mil/mobroute/api/apirtdatabaseq"
			"git.sr.ht/~mil/mobroute/api/apirtroute"
			"git.sr.ht/~mil/mobroute/api/apirtschedule"
			"git.sr.ht/~mil/mobroute/api/apirtstops"
		)
	'

	publicexportsdir apirtinit
	publicexportsdir apirtdatabase
	publicexportsdir apirtdatabaseq
	publicexportsdir apirtstops
	publicexportsdir apirtschedule
	publicexportsdir apirtroute
}

generate | gofmt
